@@ -387,7 +387,7 @@ class DefinitionJsonDeserializer:
 			dependent_definition_profile = self.deserialize(root, kw['definition_name'])
 			Index.register_definition(**dependent_definition_profile)
 		self._set(obj, kw, 'ignored')
-		self._set(obj, kw, 'input_name')
+		self._set(obj, kw, 'input_name', default_value=name)
 		self._set(obj, kw, 'output_name')
 		self._set(obj, kw, 'required')
 		self._set(obj, kw, 'unvalidated')
