@@ -70,7 +70,7 @@ class Schema:
 					if isinstance(field, Field):
 						model[entry] = field.to_json_dict(entry)
 					elif field is None:
-						field = Field()
+						field = Field(input_name=entry)
 						model[entry] = field.to_json_dict(entry)
 		result = {
 				'model_type': model_type
@@ -94,7 +94,7 @@ class Field:
 
 	def __init__(
 				self,
-				input_name=Utils.Empty,
+				input_name,
 				output_name=Utils.Empty,
 				definition_name=Utils.Empty,
 				required=Utils.Empty,
@@ -113,6 +113,10 @@ class Field:
 		_set_member('ignored', ignored)
 		_set_member('unvalidated', unvalidated)
 		_set_member('default_value', default_value)
+		if self.input_name is None:
+			raise Exceptions.InternalError('Field', 'Input name must be specified')
+		if self.output_name is None:
+			self.output_name = self.input_name
 
 	# Broken
 	def to_json_dict(self, field_name):
@@ -223,6 +227,7 @@ class ModelValidator:
 		self.default_values = {}
 		self.input_field_display_names = {}
 		self.output_names = {}
+		self.output_value_names = {}
 		self.dependency_order = []
 		self.load_schema()
 		self.name = schema.model_name
@@ -246,6 +251,7 @@ class ModelValidator:
 
 	def add_output_names(self, output_names):
 		self.output_names.update(output_names)
+		self.output_value_names.update({k:output_names[k] for k in output_names})
 
 	def add_input_names(self, input_names):
 		normalized = input_names
@@ -326,7 +332,7 @@ class ModelValidator:
 		for name in fields:
 			field = fields[name]
 			if field is None:
-				field = Field()
+				field = Field(input_name=name)
 			field_definition_name = name
 			if field.definition_name is not None:
 				field_definition_name = field.definition_name
@@ -375,7 +381,12 @@ class ModelValidator:
 			return self.input_field_display_names[field_name]
 		return field_name
 
-	def _list_input_field_names(self, vector):
+	def get_field_output_name(self, field_name):
+		if field_name in self.output_names:
+			return self.output_names[field_name]
+		return field_name
+
+	def _list_field_input_names(self, vector):
 		result = []
 		if isinstance(vector, dict):
 			for k in vector:
@@ -404,10 +415,14 @@ class ModelValidator:
 					pass
 		return result
 
-	def _get_input_value(self, fqn, field_name, source):
+	def _get_input_value_name(self, fqn, field_name):
 		value_name = field_name
 		if value_name in self.input_field_names:
 			value_name = self.input_field_names[field_name]
+		return value_name
+
+	def _get_input_value(self, fqn, field_name, source):
+		value_name = self._get_input_value_name(fqn, field_name)
 		if isinstance(source, dict):
 			if value_name in source:
 				return source[value_name]
@@ -468,7 +483,7 @@ class ModelValidator:
 		normalized = values
 		if isinstance(values, dict):
 			normalized = Parameters.Index(values)
-		field_names = self._list_input_field_names(normalized)
+		field_names = self._list_field_input_names(normalized)
 		kw = Parameters.Index()
 		missing = set(self.required)
 		# Build kw
@@ -532,12 +547,12 @@ class ModelValidator:
 					)
 		# Output Names
 		##############
-		result_keys = self._list_input_field_names(result)
-		for entry in result_keys:
-			if entry in self.output_names:
-				output_name = self.output_names[entry]
-				self._rename_output(entry, output_name, result)
-
+		result_keys = self._list_field_input_names(result)
+		for field_name in result_keys:
+			input_value_name = self._get_input_value_name(fqn, field_name)
+			if field_name in self.output_names:
+				output_name = self.output_value_names[field_name]
+				self._rename_output(input_value_name, output_name, result)
 		return result
 
 	def __call__(self, validated, fqn, field_name, definition, values, cb, **rules_kw):
